@@ -213,25 +213,44 @@ public:
             typedef typename point_quad_tree<_Key,_Data>::data_type data_type;
 
         public:
+            // Iterator traits
+            typedef ::std::pair<point, data_type> value_type;
+            typedef value_type*     pointer;
+            typedef value_type&     reference;
+            typedef ptrdiff_t       difference_type;
+            typedef ::std::bidirectional_iterator_tag iterator_category;
+
             const_iterator(res_nodes_ptr& ptr) : mp_res_nodes(ptr), m_end_pos(false) {}
 
             const_iterator(const const_iterator& r) :
                 mp_res_nodes(r.mp_res_nodes),
                 m_cur_pos(r.m_cur_pos),
+                m_cur_value(r.m_cur_value),
                 m_end_pos(r.m_end_pos) {}
 
             const_iterator& operator= (const const_iterator& r)
             {
                 mp_res_nodes = r.mp_res_nodes;
                 m_cur_pos = r.m_cur_pos;
+                m_cur_value = r.m_cur_value;
                 m_end_pos = r.m_end_pos;
                 return *this;
             }
 
             bool operator== (const const_iterator& r) const
             {
-                return mp_res_nodes.get() == r.mp_res_nodes.get() && 
-                    m_cur_pos == r.m_cur_pos && m_end_pos == r.m_end_pos;
+                if (mp_res_nodes)
+                {
+                    // Non-empty result set.
+                    return mp_res_nodes.get() == r.mp_res_nodes.get() &&
+                        m_cur_pos == r.m_cur_pos && m_end_pos == r.m_end_pos;
+                }
+
+                // Empty result set.
+                if (r.mp_res_nodes)
+                    return false;
+
+                return m_end_pos == r.m_end_pos;
             }
 
             bool operator!= (const const_iterator& r) const
